import matplotlib.pyplot as plt
from numpy import loadtxt, linspace, polyfit
from math import log, sqrt, exp

def invert_input():
    file = loadtxt('data/invert_input.txt')
    U1 = file[:,0]
    UO = file[:,1] * (-1)
    x = linspace(min(U1), max(U1))
    
    with open('data/invert_inpurt_tex.txt', 'w') as file:
        for i in range(len(U1)):
            file.write(str(U1[i]) + ' & ' + str(UO[i]) + '\\\[.5mm]\n')
    
    plt.figure(figsize=(7,5))
    plt.plot(U1, UO, 'o', label='measurement')
    plt.plot(x, -2 * x, label='$- \, R_2 \, / \, R_1 \cdot U_1$')
    plt.xlabel('$U_1$ [V]', size=13)
    plt.ylabel('$U_O$ [V]', size=13)
    plt.legend(prop={'size': 12})
    plt.savefig('invert_input.png', dpi=120)
    
invert_input()

def invert_input_bandwidth():
    file = loadtxt('data/noninvert_input.txt')
    f = file[:,0]
    Uin = file[:,1]
    Uout = file[:,2]
    Au = Uout / Uin
    
    log_f = []
    for i in range(len(f)):
        log_f.append(log(f[i], 10))
    line = max(Au) / sqrt(2)   
         
    x = linspace(min(log_f), max(log_f))
    x2 = linspace(4.03, 6, 150)
    A1, A0 = polyfit(log_f[6:8], Au[6:8], 1)
    
    with open('data/invert_inpurt_bandwidth_tex.txt', 'w') as file:
        for i in range(len(Uin)):
            file.write(str(Uin[i]) + ' & ' + str(Uout[i]) + ' & ' \
                       + str(round(log_f[i], 2)) + ' & '\
                       + str(round(Au[i], 2)) + '\\\[.5mm]\n')
    A = 3195.1
    t = 0.5469
    y0 = -0.031
    exp_x = []
    for i in x2:
        exp_x.append(y0 + A * exp(-i / t))
     
    line2 = 0
    for i in range(len(x2)):
        if round(exp_x[i], 2) == round(line, 2):
            line2 = x2[i]
    
    plt.figure(figsize=(7,5))
    plt.plot(log_f, Au, 'o')
    plt.plot(x, [line for i in range(50)])
    #plt.plot(x2, A0 + A1 * x2)
    plt.plot(x2, exp_x)
    plt.xlabel('log($\omega$)', size=13)
    plt.ylabel('$A_u$', size=13)
    plt.text(4.5, 1.65, '$A_{{u, max}} \, / \, \sqrt{{2}} = \
             {0:.2f}$\nlog($\omega$) = {1:.2f}\n$\omega = ${2:.0f} Hz'\
             .format(line, line2, 10**line2), size=12)
    plt.savefig('bandwidth.png', dpi=120)
    
invert_input_bandwidth()

def low_pass():
    file = loadtxt('data/low_pass.txt')
    f = file[:,0]
    Uin = file[:,1]
    Uout = file[:,2]
    Au = Uout / Uin
    
    log_f = []
    for i in range(len(f)):
        log_f.append(log(f[i], 10))
        
    with open('data/low_pass_tex.txt', 'w') as file:
        for i in range(len(Uin)):
            file.write(str(Uin[i]) + ' & ' + str(Uout[i]) + ' & ' \
                       + str(round(log_f[i], 2)) + ' & '\
                       + str(round(Au[i], 3)) + '\\\[.5mm]\n')
            
    A = 1289.37
    y0 = 0.001648
    t = 0.44699
    x = linspace(min(log_f[1:]), max(log_f[1:]), 150)
    x2 = linspace(min(log_f), max(log_f), 150)
    exp_x = []
    for i in x:
        exp_x.append(y0 + A * exp(-i / t))
    
    line = max(Au) / sqrt(2)    
    line2 = 0
    for i in range(len(x)):
        if round(exp_x[i], 1) == round(line, 1):
            line2 = x[i]
    
    plt.figure(figsize=(7,5))
    plt.plot(log_f, Au, 'o')
    plt.plot(x2, [line for i in range(150)])
    plt.plot(x, exp_x)
    plt.xlabel('log($\omega$)', size=13)
    plt.ylabel('$A_u$', size=13)
    plt.text(4.5, 1.55, '$A_{{u, max}} \, / \, \sqrt{{2}} = \
             {0:.2f}$\nlog($\omega$) = {1:.2f}\n$\omega = ${2:.0f} Hz'\
             .format(line, line2, 10**line2), size=12)
    plt.savefig('low_pass.png', dpi=120)
    
low_pass()

def differential_amplifier():
    file = loadtxt('data/differential_amplifier.txt')
    U1 = file[:,0]
    U2 = file[:,1]
    UO = file[:,2]
    delta_U = U2 - U1
    x = linspace(min(delta_U), max(delta_U))
    
    with open('data/differential_amplifier_tex.txt', 'w') as file:
        for i in range(len(U1)):
            file.write(str(U1[i]) + ' & ' + str(U2[i]) + ' & ' \
                       + str(UO[i]) + '\\\[.5mm]\n')
    
    plt.figure(figsize=(7,5))
    plt.plot(delta_U, UO, 'o', label='measurement')
    plt.plot(x, 2 * x, label='$2 \cdot (U_2 - U_1)$')
    plt.xlabel('$U_2 - U_1$ [V]', size=13)
    plt.ylabel('$U_O$ [V]', size=13)
    plt.legend(prop={'size': 12})
    plt.savefig('differential_amplifier.png', dpi=120)
    
differential_amplifier()
